/* ---------------------------------------------------------------------
   File:     		CELL.C - Standalone cell prep code    97-08-28
   Type:          `C' function

   Description:   Nuclear Reactor Design and Analysis.
                  Prepares reactor cell data by:
                  	- reading group cross sections from a data library
                     - collapsing the data to the required number of groups
                     - blending the cross sections as per the cell composition
                     - creating an cell output file.
  ---------------------------------------------------------------------
*/

#include <math.h>
#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <time.h>
#include <process.h>
#include <string.h>

/* required for fscanf -- file input  */
#include <io.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>

int num_mat; /* number of materials */

struct mat_db
{
	char name[5];           /* material id */
	float *sigma_a;         /* absorption X-sect for each material */
	float *sigma_f;         /* fission X-sect for each material */
	float **sigma_s;        /* scattering X-sect for each material */
	float *nusigma_f;       /* nufission X-sect for each material */
	float *sigma_tr;        /* transport X-sect for each material */
} *oldmat, *newmat;
typedef struct mat_db MATLIB;

int oldnumber; /* # of energy groups in materials library */
int newnumber; /* # of energy groups after collapsing */

struct groupsold
{
	float energy;
	float group_flux;
} *oldgroups;
typedef struct groupsold OLD;

struct groupsnew
{
	float energy;
	float sum_flux;
	int group_start;
	int group_end;
} *newgroups;
typedef struct groupsnew NEW;

int num_cel; /* number of cells */
struct cel_db
{
	char name[5];            /* cell id */
	float *sigma_a;          /* absorption x-sect for each cell */
	float *sigma_f;          /* fission x-sect for each cell */
	float *nusigma_f;        /* nufission x-sect for each cell */
	float *chi;              /* scattering x-sect for each cell */
	float *S_ext;            /* external source for each cell */
	float *sigma_tr;         /* transport x-sect for each cell */
	float *sigma_r;          /* removal x_sect for each cell */
	float **sigma_s;         /* scattering x-sect for each cell */
} *cel;
typedef struct cel_db CELLIB;

/* file names */
char mat_filename[13], grp_filename[13];
char cel_filename[13], ini_filename[13];
char out_filename[13];

/* initialize flags */
int ini_set = 0;
int mat_set = 0;
int group_set = 0;
int cel_set = 0;

char *message_group = "Materials library must be defined first";
char *message_cel = "Required energy groups must be defined first";

/* Function prototypes */
int config_input(char *filename);
int material_input(char *filename);
int group_collapse(char *filename);
int cell_prep(char *filein, char *fileout);
void post_message(char *message);

/* Constants */
#define PI      3.141593

/* -------------------------main routine---------------------------------- */
int main(int argc, char *argv[])
{
/* Initialize */

	strcpy(ini_filename, "cell.ini");
	strcpy(out_filename, "cell.out");
	if (argc > 1 && strlen(argv[1]) < 13) strcpy (ini_filename, argv[1]);
	if (argc > 2 && strlen(argv[2]) < 13) strcpy (out_filename, argv[2]);

	/* Initialize data sets */
   ini_set = 0;
   ini_set = config_input (ini_filename);
   if (ini_set != 1)
   {
	  printf("filename %s invalid", ini_filename);
     exit(0);
   }
   mat_set = 0;
   mat_set = material_input(mat_filename);
   if (mat_set != 1)
   {
	  printf("filename %s invalid", mat_filename);
     exit(0);
   }
   group_set = 0;
   group_set = group_collapse(grp_filename);
   if (group_set != 1)
   {
     printf("filename %s invalid", grp_filename);
	  exit(0);
   }
   cel_set = 0;
   cel_set = cell_prep(cel_filename, out_filename);
   if (cel_set != 1)
	{
     printf("filename %s or %s invalid", cel_filename, cel_filename);
     exit(0);
   }
   printf("done");
   return(1);
}
/*------------------------------------------------------------------------*/

/*------------------LOAD CONFIGURE FILE -------------------------------------*/

int config_input(char *filename)
{
	char buf[80];
	FILE *fp;

   /* Open an existing file for reading. Fail if file doesn't exist. */
   if( (fp = fopen( filename, "r")) != NULL )
   {
   /* Read data from file  */
	/* Title card */
	    fgets (buf, 80, fp);

	    fscanf( fp, "%s", &mat_filename);
	    fgets (buf, 80, fp);
	    fscanf( fp, "%s", &grp_filename);
	    fgets (buf, 80, fp);
		 fscanf( fp, "%s", &cel_filename);
	    fgets (buf, 80, fp);
	    fscanf( fp, "%s", &out_filename);
	    fgets (buf, 80, fp);
       fclose( fp );
       return (1); /* set flag to indicate data is setup */
    }
    else
	/* File did not exist. Print message */
	printf( "File does not exist\n" );
	return(0);
}
/*------------------------------------------------------------------------*/

/*------------INPUT MATERIAL LIBRARY DATA -----------------------------*/
/* data loading from material library */
int material_input(char *filename)
{
	char buf[120];
	FILE *fp;
	int i,j,k,diagonal, num_elements, *ivector();


   /* Open an existing file for reading. Fail if file doesn't exist. */
   if( (fp = fopen( filename, "r")) != NULL )
   {

	/* Sample Data Format

	   H                                   <--MATERIAL ID
	   1.977724 3.698452 6.602396 25.89051 <--Transport  x-sections: Groups 1,2,3,4
	   0.000000 0.000000 0.000000 0.000000 <--Fission    x-sections: Groups 1,2,3,4
	   0.000000 0.000000 0.000000 0.000000 <--Nu*Fission x-sections: Groups 1,2,3,4
		.378E-04 .186E-03 .134E-01 .2544240 <--Absorption x-sections: Groups 1,2,3,4
	   1  3  1.363900  1.595054  0.010803  <***SCATTER MATRIX (Maclib-69 Format)
	   1  3  9.335382  1.955215  0.000189  <***Each Line corresponds to one matrix row
	   1  2  18.41277  1.973774            <***First Integer:  Identifies Diagonal Element
	   2  2  0.002658  37.25573            <***Second Integer: # of elements in Matrix Row

		 For this material the 4x4 scatter matrix would be:

		     1.363900  1.595054  0.010803  0.000000
		     0.000000  9.335382  1.955215  0.000189
		     0.000000  0.000000  18.41277  1.973774
		     0.000000  0.000000  0.002658  37.25573

	*/

	/* Title card */
	   fgets (buf, 120, fp);
	/* Number of energy groups in the library */
	   fscanf( fp, "%d", &oldnumber );
	   fgets (buf, 120, fp);  /* read comment */

	/* allocate memory for library energy groups */
	   oldgroups = (OLD *) malloc ((oldnumber) * sizeof (OLD) );

	/* read energy levels and group fluxes */
	   for(i=0;i<oldnumber;i++)
	   {
		  fscanf(fp, "%f %f", &oldgroups[i].energy, &oldgroups[i].group_flux);
	     fgets(buf, 120, fp);
	   }

	/* number of elements in library */
	   fscanf(fp, "%d", &num_mat);
	   fgets(buf, 120, fp);


	/* allocate memory for the structure vector */
	   oldmat = (MATLIB *) malloc (num_mat * sizeof ( MATLIB) );
	   for(i=0;i<num_mat;i++)
	   {
		  oldmat[i].sigma_tr = (float *) malloc (oldnumber * sizeof (float));
	     oldmat[i].sigma_a = (float *) malloc (oldnumber * sizeof (float));
	     oldmat[i].sigma_f = (float *) malloc (oldnumber * sizeof (float));
	     oldmat[i].nusigma_f = (float *) malloc (oldnumber * sizeof (float));
	     oldmat[i].sigma_s = (float **) malloc (oldnumber * sizeof (float));
	     for(j=0;j<oldnumber;j++)
	     {
			 oldmat[i].sigma_s[j] = (float *) malloc (oldnumber * sizeof (float));
	     }
	   }

	/* sample data format */
	   for(i=0;i<12;i++)
	   {
	     fgets(buf, 120, fp);
		}

	/* read material x-sections */
	   for(i=0;i<num_mat;i++)
	   {
		  fscanf(fp, "%s", oldmat[i].name);
	     fgets(buf, 120, fp);
	     for(j=0;j<oldnumber;j++)
	     {
	       fscanf(fp, "%f", &oldmat[i].sigma_tr[j]);
	     }
	     fgets(buf, 120, fp);
		  for(j=0;j<oldnumber;j++)
	     {
	       fscanf(fp, "%f", &oldmat[i].sigma_f[j]);
	     }
	     fgets(buf, 120, fp);
	     for(j=0;j<oldnumber;j++)
	     {
	       fscanf(fp, "%f", &oldmat[i].nusigma_f[j]);
		  }
	     fgets(buf, 120, fp);
	     for(j=0;j<oldnumber;j++)
	     {
	       fscanf(fp, "%f", &oldmat[i].sigma_a[j]);
		  }
	     fgets(buf, 120, fp);


	     /* read scattering x-sections and setup scatter matrix */
	     for(j=0;j<oldnumber;j++)
	     {
			 for(k=0;k<oldnumber;k++)
	       {
		 oldmat[i].sigma_s[j][k]=0.0;
	       }
	     }
	     for(j=0;j<oldnumber;j++)
	     {
	       fscanf(fp, "%d %d", &diagonal, &num_elements);
			 for(k=0;k<num_elements;k++)
	       {
		 fscanf(fp, "%f", &oldmat[i].sigma_s[j][j+1-diagonal+k]);
	       }
	       fgets(buf, 120, fp);
		  }
	   }

	   fclose( fp );
	   return (1); /* set flag to indicate data is setup */
    }
    else        /* File did not exist. Print message */
	 {
	  post_message("Could not open the materials library file");
	  return(0);
	 }
}

/*-----------------------------------------------------------------------*/

/*------------GROUP COLLAPSING TO REQUIRED GROUPS------------------------*/
/* input energy groups desired from file and group collapse */
int group_collapse(char *filename)
{
	int p,k,i,j,a,b;
	float sum;
	char buf[120];
	FILE *fp;

	if (mat_set ==0)
	{
	   post_message(message_group);
		return(0);
	}

   /* Open an existing file for reading. Fail if file doesn't exist. */
   if( (fp = fopen( filename, "r")) != NULL )
   {

	  /* Title Card */
	  fgets(buf, 120, fp);
	  /* Number of required groups */
	  fscanf(fp, "%d", &newnumber);

	  if(newnumber>oldnumber)         /* check validity of group input */
	  {
	    printf("Number of required groups > that specified in material library");
	    fclose(fp);
       getch();
	    return(0);
     }
	  /* read comments */
	  fgets(buf, 120, fp);

	  /* allocate memory for required energy groups */
     newgroups = (NEW *) malloc (newnumber * sizeof (NEW) );

     /* allocate memory for the structure vector */
     newmat = (MATLIB *) malloc (num_mat * sizeof ( MATLIB) );
     for(i=0;i<num_mat;i++)
	  {
	     newmat[i].sigma_tr = (float *) malloc (newnumber * sizeof (float));
	     newmat[i].sigma_a = (float *) malloc (newnumber * sizeof (float));
	     newmat[i].sigma_f = (float *) malloc (newnumber * sizeof (float));
	     newmat[i].nusigma_f = (float *) malloc (newnumber * sizeof (float));
		  newmat[i].sigma_s = (float **) malloc (newnumber * sizeof (float));
	     for(j=0;j<newnumber;j++)
	     {
	       newmat[i].sigma_s[j] = (float *) malloc (newnumber * sizeof (float));
	     }
     }

	  /* required energy levels */
     for(i=0;i<newnumber;i++)
     {
	     fscanf(fp, "%f", &newgroups[i].energy);
	     fgets(buf, 120, fp);
     }

	  for(i=0;i<num_mat;i++)        /* material names do not change */
	  {
	    strcpy(newmat[i].name, oldmat[i].name);
	  }


	  if(newnumber==oldnumber)      /* no collapsing necessary */
	  {
	    for(i=0;i<num_mat;i++)
	    {
	      newmat[i] = oldmat[i];
	    }
	    fclose(fp);
		 return(1);
     }

	  if(newnumber==1)         /* special case of one speed model */
	  {                        /* scattering x-sections not necessary */
	    sum = 0.0;
	    for(j=0;j<oldnumber;j++)
	    {
			sum = sum + oldgroups[j].group_flux;
       }
	    for(i=0;i<num_mat;i++)
	    {
	      newmat[i].sigma_a[0] = 0.0;
			newmat[i].sigma_f[0] = 0.0;
	      newmat[i].nusigma_f[0] = 0.0;
	      newmat[i].sigma_tr[0] = 0.0;
	      for(j=0;j<oldnumber;j++)
	      {
	        newmat[i].sigma_a[0] = newmat[i].sigma_a[0]
         					+ oldmat[i].sigma_a[j] * oldgroups[j].group_flux;
			  newmat[i].sigma_f[0] = newmat[i].sigma_f[0]
         					+ oldmat[i].sigma_f[j] * oldgroups[j].group_flux;
	        newmat[i].sigma_tr[0] = newmat[i].sigma_tr[0]
         					+ oldmat[i].sigma_tr[j] * oldgroups[j].group_flux;
           newmat[i].nusigma_f[0] = newmat[i].nusigma_f[0]
         					+ oldmat[i].nusigma_f[j] * oldgroups[j].group_flux;
         }
	      newmat[i].sigma_a[0] = newmat[i].sigma_a[0]/sum;
			newmat[i].sigma_f[0] = newmat[i].sigma_f[0]/sum;
	      newmat[i].nusigma_f[0] = newmat[i].nusigma_f[0]/sum;
	      newmat[i].sigma_tr[0] = newmat[i].sigma_tr[0]/sum;
       }
	    fclose(fp);
		 return(1);
     }

	  /* convert energy divisions to group indices */
	  p=0;
	  for(k=0;k<oldnumber;k++)
	  {
		 if(newgroups[p].energy<=oldgroups[k].energy)
	    {
	      if(newgroups[p].energy>oldgroups[k+1].energy)
	      {
	        newgroups[p].group_start = k;
	        p++;
	      }
       }
		 else if(newgroups[p].energy>oldgroups[k].energy)
	    {
	      newgroups[p].group_start = k;
	      p++;
       }
		 if(p==newnumber) k=oldnumber;
     }
	  newgroups[newnumber-1].group_end = oldnumber-1;
	  for(k=0;k<(newnumber-1);k++)
	  {
	    newgroups[k].group_end = newgroups[k+1].group_start-1;
     }

	  /* calculate the intragroup fluxes */
	  for(p=0;p<newnumber;p++)
	  {
	    newgroups[p].sum_flux = 0.0;
	    for(k=newgroups[p].group_start;k<(newgroups[p].group_end+1);k++)
	    {
	      newgroups[p].sum_flux = newgroups[p].sum_flux + oldgroups[k].group_flux;
		 }
     }

	  /* group collapsing scatter algorithm */
	  for(i=0;i<num_mat;i++)
	  {
	    for(a=0;a<newnumber;a++)
	    {
	      for(b=0;b<newnumber;b++)
	      {
	        newmat[i].sigma_s[a][b] = 0.0;
	        for(p=newgroups[a].group_start;p<(newgroups[a].group_end+1);p++)
			  {
		       for(k=newgroups[b].group_start;k<(newgroups[b].group_end+1);k++)
		       {
               newmat[i].sigma_s[a][b]=newmat[i].sigma_s[a][b]
         		       +oldmat[i].sigma_s[p][k]*oldgroups[p].group_flux;
             }
           }
           newmat[i].sigma_s[a][b] = newmat[i].sigma_s[a][b]/newgroups[a].sum_flux;
			}
		 }
	  }


	  /* group collapsing for absortion, fission, transport..etc. */
	  for(i=0;i<num_mat;i++)
	  {
		 for(a=0;a<newnumber;a++)
		 {
			newmat[i].sigma_a[a] = 0.0;
			newmat[i].sigma_f[a] = 0.0;
			newmat[i].sigma_tr[a] = 0.0;
			newmat[i].nusigma_f[a] = 0.0;
			for(k=newgroups[a].group_start;k<(newgroups[a].group_end+1);k++)
			{
			  newmat[i].sigma_a[a]=newmat[i].sigma_a[a]
								+oldmat[i].sigma_a[k]*oldgroups[k].group_flux;
			  newmat[i].sigma_f[a]=newmat[i].sigma_f[a]
								+oldmat[i].sigma_f[k]*oldgroups[k].group_flux;
			  newmat[i].sigma_tr[a]=newmat[i].sigma_tr[a]
								+oldmat[i].sigma_tr[k]*oldgroups[k].group_flux;
			  newmat[i].nusigma_f[a]=newmat[i].nusigma_f[a]
								+oldmat[i].nusigma_f[k]*oldgroups[k].group_flux;
			}
			newmat[i].sigma_a[a] = newmat[i].sigma_a[a]/newgroups[a].sum_flux;
			newmat[i].sigma_f[a] = newmat[i].sigma_f[a]/newgroups[a].sum_flux;
			newmat[i].sigma_tr[a] = newmat[i].sigma_tr[a]/newgroups[a].sum_flux;
			newmat[i].nusigma_f[a] = newmat[i].nusigma_f[a]/newgroups[a].sum_flux;
		 }
	  }

	  fclose( fp );
	  return (1); /* set flag to indicate data is setup */
	}
	else        /* File did not exist. Print message */
	{
	  post_message("Could not open the energy group file");
	  return(0);
	}
}

/*----------------------------------------------------------------------*/


/*------------INPUT CELL LIBRARY DATA -----------------------------------*/
/* data loading from cell definition library, preparation of data and output*/
int cell_prep(char *filein, char *fileout)
{
  char buf[100], mat_name[5];
  FILE *fp, *fpout;
  int i, j, k, p, q, num_elements, mat_id, *ivector();
  float volume, nuclei, energy_avg, *v;

  if (group_set == 0)
  {
	 post_message(message_cel);
	 return(0);
  }

  /* Open a file for writing. Fail if file doesn't exist. */
  if( (fpout = fopen( fileout, "w")) != NULL )
  {

	/* Open an existing file for reading. Fail if file doesn't exist. */
	if( (fp = fopen( filein, "r")) != NULL )
	{
	  if( (fpout = fopen( fileout, "w")) != NULL )
	  {

		/* Read data from file  */
		/* Example format:
		Cell definition library
		2       <- number of cells in the following list
		ID   # of elements in the cell
		FUEL 3  <- fuel cell consisting of natural uranium and light water
			 U235     2.00  0.0478<- Uranium 235: 2% by vol, nuclei per unit vol * E-24
			 U238     8.00  0.0478<- Uranium 238: 8% by vol, nuclei per unit vol * E-24
			 H2O     90.00  0.0335<- light water:90% by vol, nuclei per unit vol * E-24
		MOD  2  <- light water moderator
			 H2O    99.985  0.0335<- light water
			 D20     0.015  0.0331<- heavy water
		*/

		  /* Title card */
		  fgets (buf, 80, fp);
		  fprintf(fpout,"Cell output cross sections\n");
		  fprintf(fpout, "%d   <--# of energy groups\n", newnumber);
		  v = (float *) malloc (newnumber * sizeof ( float) );
		  for (p=0;p<newnumber;p++)
		  {
		  if (p==newnumber) energy_avg = newgroups[p].energy / 2.0;
		  energy_avg = (newgroups[p].energy + newgroups[p+1].energy)/2.0;
		  v[p] = 220000.0*pow(energy_avg/0.025, 0.5);
		  fprintf (fpout,"%g ",v[p]);
		  }
		  fprintf(fpout, " <--velocities of groups\n");
		  /* Number of elements in the library */
		  fscanf( fp, "%d", &num_cel );
		  fgets (buf, 100, fp); /* read comments */
		  fprintf(fpout, "%d %s", num_cel, buf);
		  /* label card */
		  fgets (buf, 100, fp); /* read comment */
		  fprintf(fpout, "%s", buf);

		  /* allocate memory for the structure vector */
		  cel = (CELLIB *) malloc (num_cel * sizeof ( CELLIB) );
		  for(i=0;i<num_cel;i++)
		  {
			 cel[i].sigma_a = (float *) malloc (newnumber * sizeof (float));
			 cel[i].sigma_f = (float *) malloc (newnumber * sizeof (float));
			 cel[i].nusigma_f = (float *) malloc (newnumber * sizeof (float));
			 cel[i].chi = (float *) malloc (newnumber * sizeof (float));
			 cel[i].S_ext = (float *) malloc (newnumber * sizeof (float));
			 cel[i].sigma_tr = (float *) malloc (newnumber * sizeof (float));
			 cel[i].sigma_r = (float *) malloc (newnumber * sizeof (float));
			 cel[i].sigma_s = (float **) malloc (newnumber * sizeof (float));
			 for(j=0;j<newnumber;j++)
			 {
				cel[i].sigma_s[j] = (float *) malloc (newnumber * sizeof (float));
			 }
		  }


		  /* read cell id and the number of elements in the cell */
		  for (i = 0; i < num_cel; i++)
		  {
			 fscanf( fp, "%s %d", &cel[i].name, &num_elements );
			 fgets (buf, 80, fp); /* read comments */
			 fprintf(fpout, "%s %s", cel[i].name, buf);
			 for(j=0;j<newnumber;j++)
			 {
				cel[i].sigma_a[j] = 0.0;
				cel[i].sigma_f[j] = 0.0;
				cel[i].nusigma_f[j] = 0.0;
				cel[i].sigma_tr[j] = 0.0;
				cel[i].sigma_r[j] = 0.0;
			 }
			 for(p=0;p<newnumber;p++)
			 {
				for(q=0;q<newnumber;q++)
				{
				 cel[i].sigma_s[p][q] = 0.0;
				}
			 }

			 for (j = 0; j < num_elements; j++)
			 {
				fscanf( fp, "%s %f %f", &mat_name, &volume, &nuclei);
				fgets (buf, 100, fp); /* read comment */
				mat_id = -1;
				for (k = 0; k < num_mat; k++)
				{
				  if (!strcmp (newmat[k].name, mat_name) )
				{
				  mat_id = k;
				  break;
				}
			 }
			 if (mat_id < 0 )
			 {
				 printf ("Cell %d definition contains an unknown element %d", i,j);
				 getch();
				 return(0);
			 }
			 /* calculate macroscopic cell-averaged x-sections */
			 for(p=0;p<newnumber;p++)
			 {
				cel[i].sigma_a[p] = cel[i].sigma_a[p] + newmat[mat_id].sigma_a[p] *
										nuclei * volume / 100.0;
				cel[i].sigma_f[p] = cel[i].sigma_f[p] + newmat[mat_id].sigma_f[p] *
										nuclei * volume / 100.0;
				cel[i].nusigma_f[p] = cel[i].nusigma_f[p] +
								newmat[mat_id].nusigma_f[p] * nuclei * volume / 100.0;
				cel[i].sigma_tr[p] = cel[i].sigma_tr[p] +
								newmat[mat_id].sigma_tr[p] * nuclei * volume /100.0;
			 }

			 /* calculate macroscopic cell-averaged scatter x-sections */
			 for(p=0;p<newnumber;p++)
			 {
				for(q=0;q<newnumber;q++)
				{
				  cel[i].sigma_s[p][q] = cel[i].sigma_s[p][q] +
						  newmat[mat_id].sigma_s[p][q] * nuclei * volume /100.0;
				}
			 }
		  }

		  /* calculate removal x-sections */
		  if(newnumber==1)
		  {
		  cel[i].sigma_r[0] = cel[i].sigma_a[0];
		  }
		  else
		  {
			 for(p=0;p<newnumber;p++)
			 {
				for(q=0;q<newnumber;q++)
				{
				 cel[i].sigma_r[p] = cel[i].sigma_r[p] + cel[i].sigma_s[p][q];
				}
				cel[i].sigma_r[p] = cel[i].sigma_r[p] + cel[i].sigma_a[p];
				cel[i].sigma_r[p] = cel[i].sigma_r[p] - cel[i].sigma_s[p][p];
			 }
		  }
		  /* Currently the materials library does not provide the distribution of
		  fission neutrons.  So, assume that all fission neutrons are produced in
		  the highest energy group.  This is quite reasonable for "few group" core
			codes such as this one.  This parameter is only relevant when nusigma_f is
		  non-zero.
		  */
		  for(p=1;p<newnumber;p++)
		  {
			 cel[i].chi[p]=0.0;
		  }
		  cel[i].chi[0]=1.0;
		  /* The materials library does contain any sources.  Zero entries are
		  provided so the user can easily add them manually to the output file.
		  This is handy for assignments.
		  */
		  for(p=0;p<newnumber;p++)
		  {
			 cel[i].S_ext[p]=0.0;
		  }

		  for(p=0;p<newnumber;p++)
		  {
			 fprintf(fpout, "%10.5g ", cel[i].sigma_a[p]);
		  }
		  fprintf(fpout, " <-- sigma_a for groups 1 -> %d\n", newnumber);
		  for(p=0;p<newnumber;p++)
		  {
			 fprintf(fpout, "%10.5g ", cel[i].sigma_f[p]);
		  }
		  fprintf(fpout, " <-- sigma_f for groups 1 -> %d\n", newnumber);
		  for(p=0;p<newnumber;p++)
		  {
			 fprintf(fpout, "%10.5g ", cel[i].nusigma_f[p]);
		  }
		  fprintf(fpout, " <-- nu*sigma_f for groups 1 -> %d\n", newnumber);
		  for(p=0;p<newnumber;p++)
		  {
			 fprintf(fpout, "%10.5g ", cel[i].sigma_tr[p]);
		  }
		  fprintf(fpout, " <-- sigma_tr for groups 1 -> %d\n", newnumber);
		  for(p=0;p<newnumber;p++)
		  {
			 fprintf(fpout, "%10.5g ", cel[i].sigma_r[p]);
		  }
		  fprintf(fpout, " <-- sigma_r for groups 1 -> %d\n", newnumber);

		  for(p=0;p<newnumber;p++)
		  {
			 for(q=0;q<newnumber;q++)
			 {
				fprintf(fpout, "%10.5g ", cel[i].sigma_s[p][q]);
			 }
			 fprintf(fpout, " <-- sigma_s (%d->%d)\n", p+1, newnumber);
		  }

		  for(p=0;p<newnumber;p++)
		  {
			 fprintf(fpout, "%10.5g ", cel[i].chi[p]);
		  }
		  fprintf(fpout, " <-- chi for groups 1 -> %d\n", newnumber);

		  for(p=0;p<newnumber;p++)
		  {
			 fprintf(fpout, "%10.5g ", cel[i].S_ext[p]);
		  }
		  fprintf(fpout, " <-- S_ext for groups 1 -> %d\n", newnumber);

		}
		fclose( fp );
      return (1); /* set flag to indicate data is setup */
     }
     else	/* File did not exist. Print message */
     {
       post_message("Could not open the cell definition file");
	    return(0);
     }
   }
   else	/* File did not exist. Print message */
   {
      post_message("Could not open the cell definition file");
	   return(0);
   }
  }
  return(1);
}
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* post message box */
void post_message ( char *message)
{
  printf("%s",message);
}



